@@ -1,11 +1,12 @@
 import python_jsonschema_objects.util as util
 import python_jsonschema_objects.validators as validators
 
+import collections
 import itertools
 import six
 
 
-class ProtocolBase(object):
+class ProtocolBase( collections.MutableMapping):
     __propinfo__ = {}
     __required__ = set()
 
@@ -15,10 +16,22 @@ class ProtocolBase(object):
         'integer': int,
         'number': float,
         'null': None,
-        'string': six.text_type,
+        'string': six.string_types,
         'object': dict
     }
 
+    def as_dict(self):
+      out = {}
+      for prop in self:
+          propval = getattr(self, prop)
+
+          if isinstance(propval, list):
+              out[prop] = [x.as_dict() for x in propval]
+          elif propval is not None:
+              out[prop] = propval.as_dict()
+
+      return out
+
     def __str__(self):
         return repr(self)
 
@@ -65,17 +78,28 @@ class ProtocolBase(object):
               "Attempted to set unknown property '{0}', but 'additionalProperties' is false.")
         else:
           typ = getattr(self, '__extensible__', None)
-          if isinstance(typ, type) and issubclass(typ, LiteralValue):
+          if typ is True:
+            # There is no type defined, so just make it a basic literal
+            # Pick the type based on the type of the values
+            valtype = [k for k, t in self.__SCHEMA_TYPES__.iteritems()
+                       if t is not None and isinstance(val, t)][0]
+            val = MakeLiteral(name, valtype, val)
+          elif isinstance(typ, type) and issubclass(typ, LiteralValue):
             val = typ(val)
           elif isinstance(typ, type) and issubclass(typ, ProtocolBase):
             val = typ(**val)
 
         self._extended_properties[name] = val
 
+    """ Implement collections.MutableMapping methods """
+
     def __iter__(self):
       import itertools
-      return itertools.chain(self._extended_properties.iteritems(),
-                             self._properties.iteritems())
+      return itertools.chain(self._extended_properties.iterkeys(),
+                             self._properties.iterkeys())
+
+    def __len__(self):
+      return len(self._extended_properties) + len(self._properties)
 
     def __getitem__(self, key):
       return self.__getattr__(key)
@@ -83,6 +107,9 @@ class ProtocolBase(object):
     def __setitem__(self, key, val):
       return self.__setattr__(key, val)
 
+    def __delitem__(self, key):
+      return self.__delattr__(key)
+
     def __getattr__(self, name):
       if name not in self._extended_properties:
         raise AttributeError("{0} is not a valid property of {1}".format(
@@ -101,51 +128,47 @@ class ProtocolBase(object):
         enc = util.ProtocolJSONEncoder()
         return enc.encode(self)
 
-    def validate(this):
-        missing = [x for x in this.__required__
-                   if this._properties[x] is None]
+    def validate(self):
+        missing = [x for x in self.__required__
+                   if x not in self._properties or self._properties[x] is None]
 
         if len(missing) > 0:
             raise validators.ValidationError(
                 "'{0}' are required attributes for {1}"
-                            .format(missing, this.__class__))
+                            .format(missing, self.__class__))
 
-        for prop, val in this._properties.iteritems():
+        for prop, val in self._properties.iteritems():
             if val is None:
                 continue
 
-            this.validate_property(prop, val)
+            if isinstance(val, ProtocolBase):
+                val.validate()
+            elif isinstance(val, LiteralValue):
+                val.validate()
+            elif isinstance(val, list):
+                for subval in val:
+                  subval.validate()
+            else:
+                # This object is of the wrong type, but just try setting it
+                # The property setter will enforce its correctness
+                # and handily coerce its type at the same time
+                setattr(self, prop, val)
 
         return True
 
-    def validate_property(self, prop, val):
-        """Validate a property value, and return true or false
+def MakeLiteral(name, typ, value, **properties):
+      properties.update({'type': typ})
+      klass =  type(str(name), tuple((LiteralValue,)), {
+        '__propinfo__': { '__literal__': properties}
+        })
 
-        :propinfo: A dictionary containing property info
-        :propval: The property value
-        :returns: True or False
-        """
-        info = self.propinfo(prop)
-
-        for param, paramval in info.iteritems():
-            validator = getattr(validators, param, None)
-            if validator is not None:
-                if param == 'minimum':
-                    validator(paramval, val,
-                              info.get('exclusiveMinimum',
-                                       False))
-                elif param == 'maximum':
-                    validator(paramval, val,
-                              info.get('exclusiveMaximum',
-                                       False))
-                else:
-                    validator(paramval, val)
+      return klass(value)
 
 
-class LiteralValue(ProtocolBase):
+class LiteralValue(object):
   """Docstring for LiteralValue """
 
-  def __init__(self, value):
+  def __init__(self, value, typ=None):
       """@todo: to be defined
 
       :value: @todo
@@ -154,6 +177,20 @@ class LiteralValue(ProtocolBase):
       self._value = value
       self.validate()
 
+  def as_dict(self):
+      return self._value
+
+  @classmethod
+  def propinfo(cls, propname):
+      if propname not in cls.__propinfo__:
+          return {}
+      return cls.__propinfo__[propname]
+
+  def serialize(self):
+      self.validate()
+      enc = util.ProtocolJSONEncoder()
+      return enc.encode(self)
+
   def __repr__(self):
       return "<%s %s>" % (
           self.__class__.__name__,
@@ -161,7 +198,31 @@ class LiteralValue(ProtocolBase):
       )
 
   def validate(self):
-      self.validate_property('__literal__', self._value)
+      info = self.propinfo('__literal__')
+
+      for param, paramval in info.iteritems():
+          validator = getattr(validators, param, None)
+          if validator is not None:
+              if param == 'minimum':
+                  validator(paramval, self._value,
+                            info.get('exclusiveMinimum',
+                                     False))
+              elif param == 'maximum':
+                  validator(paramval, self._value,
+                            info.get('exclusiveMaximum',
+                                     False))
+              else:
+                  validator(paramval, self._value)
+
+  def __cmp__(self, other):
+    if isinstance(other, six.integer_types):
+      return cmp(int(self), other)
+    elif isinstance(other, six.string_types):
+      return cmp(str(self), other)
+    elif isinstance(other, float):
+      return cmp(float(self), other)
+    else:
+      return cmp(id(self), id(other))
 
   def __int__(self):
     return int(self._value)
@@ -276,7 +337,7 @@ class ClassBuilder(object):
       :returns: @todo
 
       """
-      cls = type(str(nm.split('/')[-1]), tuple((LiteralValue,)), {
+      cls = type(str(nm), tuple((LiteralValue,)), {
         '__propinfo__': { '__literal__': clsdata}
         })
 
@@ -382,8 +443,10 @@ class ClassBuilder(object):
             else:
                 desc = detail[
                     'description'] if 'description' in detail else ""
+                uri = "{0}/{1}".format(nm, prop)
+                typ = self.construct(uri, detail)
 
-                props[prop] = make_property(prop, detail, desc)
+                props[prop] = make_property(prop, {'type': typ}, desc)
 
         """ If this object itself has a 'oneOf' designation, then
         make the validation 'type' the list of potential objects.
@@ -421,7 +484,13 @@ class ClassBuilder(object):
 
         if 'required' in clsdata:
             for prop in clsdata['required']:
-                required.add(prop.replace('@', ''))
+                required.add(prop)
+
+        invalid_requires = [req for req in required if req not in props['__propinfo__']]
+        if len(invalid_requires) > 0:
+          raise validators.ValidationError("Schema Definition Error: {0} schema requires "
+                                           "'{1}', but properties are not defined"
+                                           .format(nm, invalid_requires))
 
         props['__required__'] = required
 
@@ -465,18 +534,20 @@ def make_property(prop, info, desc=""):
 
         elif info['type'] == 'array':
             instance = info['validator'](val)
-            instance.validate()
+            val = instance.validate()
 
-        elif (info['type'] in this.__SCHEMA_TYPES__.keys() and val is not None):
-            val = this.__SCHEMA_TYPES__[info['type']](val)
+        elif issubclass(info['type'], LiteralValue):
+            if not isinstance(val, info['type']):
+                val = info['type'](val)
 
         elif issubclass(info['type'], ProtocolBase):
             if not isinstance(val, info['type']):
                 val = info['type'](**val)
 
             val.validate()
+        else:
+            raise TypeError("Unknown object type: '{0}'".format(info['type']))
 
-        this.validate_property(prop, val)
         this._properties[prop] = val
 
     def delprop(this):
